package functions;

use Data::Dumper;
use strict;
use Exporter;
use constant true => 1;
use constant false => 0;

my @ISA = qw(Exporter);
my @EXPORT = qw(&cmd2arr &is_avi &ask &sec2hrs &sec2hrsd &is_sub &get_file_from_path &arr2file &is_dvd &cmd2file &get_titles &get_chapters &get_audios &get_crop &get_vbitrate &get_ac3_bitrate &str2file &get_crop_width_from_str &check_crop &slash);

sub slash {
	my $string = shift;
	$string =~ s/\ /\\ /g;
	$string =~ s/\[/\\[/g;
	$string =~ s/\]/\\]/g;
	$string =~ s/\(/\\(/g;
	$string =~ s/\)/\\)/g;
	return $string
}

sub check_crop {
	my $crop = shift;
	my $result = false;
	if($crop =~ /\d{1,3}:\d{1,3}:\d{1,3}:\d{1,3}/) {
		$result = true;
	}
	return $result;
}

sub get_crop_width_from_str {
	my $crop = shift;
	my @crop = split(/:/,$crop);
	return $crop[0];
}
	

sub get_ac3_bitrate {
	my($path,$title,$aid) = @_;
	my $cmd = "mplayer -dvd-device $path dvd://$title -aid $aid -frames 0 2>/dev/null -identify";
	my @arr = &cmd2arr($cmd);
	my $ac3_bitrate;
	foreach(@arr) {
		if($_ =~ /ID_AUDIO_BITRATE=(\d+)/) {
			$ac3_bitrate = $1 / 1000;
		}
	}
	return $ac3_bitrate;
}

sub get_vbitrate {
	my($abitrate,$cd_size,$length) = @_;
	if($cd_size && $length && $abitrate) {
		my $vbitrate = int(($cd_size * 1024 * 8)/$length) - $abitrate;
		return $vbitrate;
	}
}

sub get_crop {
	my $cmd = shift;
	my @arr = &cmd2arr($cmd);
	my $radek;
	do {
		$radek = pop(@arr);
	} until($radek =~ /(\d+):(\d+):(\d+):(\d+)/);
	my @crop = ($1,$2,$3,$4);
	return @crop;
}

sub get_audios {
	my($path,$title) = @_;
	my(@audios,$radek);
	my $cmd = "lsdvd $path 2>/dev/null -t $title -a";
	my @arr = &cmd2arr($cmd);
	foreach $radek (@arr) {
		if($radek =~ /Audio: (\d+), Language: .+ - (.+), Format: (.+), Frequency: .* Channels: (\d)/) {
			$audios[$1] = $2." - ".$3." - ".$4." channels";
			if(($2 eq "cs") || ($2 eq "Czech")) {
				$audios[0] = $1;
			}
		}
		if(! $audios[0]) {
			$audios[0] = 1;
		}
	}
	return @audios;
}

sub get_chapters {
	my($path, $title) = @_;
	my(@chapters,$radek);
	my $cmd = "lsdvd 2>/dev/null -t $title -x $path -c";
	my @arr = &cmd2arr($cmd);
	foreach $radek (@arr) {
		if($radek =~ /Chapter: (\d+), Length: (\d\d:\d\d:\d\d)/) {
			$chapters[$1] = "$2";
		}
		if($radek =~ /FPS: (\d+)/) {
			$chapters[0] = $1;
		}
	}
	return @chapters;
}

sub get_titles {
	my $path = shift;
	my(@titles,$radek,$longest);
	my $cmd = "lsdvd 2>/dev/null $path";
	my @arr = &cmd2arr($cmd);
	foreach $radek (@arr) {
		if($radek =~ /^Title: (\d+), Length: (\d\d:\d\d:\d\d)/) {
			$titles[$1] = "$2";
		}
		if($radek =~ /^Longest track: (\d+)/) {
			$titles[0] = $1;
		}
			
	}
	return @titles;
}

sub is_dvd { # zjisti, jestli je na dane ceste DVD struktura
	my $path = shift;
	my $cmd = "lsdvd 2>/dev/null $path";
	my @arr = &cmd2arr($cmd);
	if(@arr > 0) {
		return 1;
	} else {
		return 0;
	}
}

sub srt2sub {
	my @srt = @_;
	print Dumper(@srt);
	my $fps = pop(@srt);
	my ($i,@sub);
	my $j = 0;
	for($i=0;$i<@srt;$i++) {
		if($srt[$i] =~ /^(\d{2}:\d{2}:\d{2},\d{3}) \-\-> (\d{2}:\d{2}:\d{2},\d{3})/) {
			my $start = int(&hrs2sec($1) * $fps);
			my $stop  = int(&hrs2sec($2) * $fps);
			my $text  = $srt[$i+1];
			if($srt[$i+2] ne "") {
					$text .= "|".$srt[$i+2];
			}
			my $sub   = "{".$start."}{".$stop."}".$text;
			$sub[$j]  = $sub;
			$j++;
		}
	}
	return @sub;
}

sub hrs2sec {
	my $hrs = shift;
	my @hrs = split(/:/,$hrs);
	my $sec = $hrs[0] * 3600 + $hrs[1] * 60 + $hrs[2];
	return $sec;
}

sub hrs2min {
	my $hrs = shift;
	my @hrs = split(/:/,$hrs);
	my $min = $hrs[0] * 60 + $hrs[1];
	return $min;
}

sub cmd2file { # prevedeme vystup prikazu do souboru
	my($cmd,$file) = @_;
	my(@arr);
	my $i = 0;
	if(open(POINT,"$cmd |") && open(FILE,">$file")) {
		while(my $radek = <POINT>) {
			print FILE $radek;
		}
		close(FILE);
		close(POINT);
		return 1;
	} else {
		print STDERR "\nCannot run '$cmd' or open '$file' fow writing\n";
		return 0;
	}
}

sub str2file {
	my ($str,$file) = @_;
	if(open(POINT,">$file")) {
		print POINT $str;
		close(POINT);
	} else {
		print STDERR "\nNemuzu otevrit soubor '$file' pro zapis\n";
		exit;
	}
}

sub arr2file {
	my @arr = @_;
	my $file = pop(@arr);
	my $radek;
	if(open(POINT,">$file")) {
		foreach $radek (@arr) {
			print POINT $radek."\n";
		}
		close(POINT);
	} else {
		print STDERR "\nNemuzu otevrit soubor '$file' pro zapis\n";
		exit;
	}
}

sub shift_sub {
	my @sub = @_;
	my $frames = pop(@sub);
	my $i = 0;
	my @arr;
	foreach my $item (@sub) {
		if($item =~ /^\{(\d+)\}\{(\d+)\}(.+)$/) {
			my $start = $1 + $frames;
			my $end = $2 + $frames;
			$arr[$i] = "{".$start."}{".$end."}".$3;
			$i++;
		}
	}
	return @arr;
}

sub get_file_from_path {
	my $path = shift;
	my @arr = split(/\//,$path);
	my $file = pop(@arr);
	return $file;
}

sub is_sub {
	my $path = shift;
	my @arr = &file2arr($path);
	if($arr[1] =~ /^\{\d+\}\{\d+\}.+/) {
		return 1;
	} else {
		return 0;
	}

}

sub is_srt {
	my $path = shift;
	my @arr = &file2arr($path);
	if(
		$arr[0] =~ /^\d+/ &&
		$arr[1] =~ /^\d{2}:\d{2}:\d{2},\d{3} \-\-> \d{2}:\d{2}:\d{2},\d{3}/
	) {
		return 1;
	} else {
		return 0;
	}
}

sub file2arr {
	my $path = shift;
	my @arr;
	my $i = 0;
	my $radek;
	if(open(POINT,$path)) {
		while($radek = <POINT>) {
			$radek =~ s/\r//g;
			$radek =~ s/\n//g;
			$arr[$i] = $radek;
			$i++;
		}
		close(POINT);
		return @arr;
	} else {
		print STDERR "\nCannot read file '$path'\n";
	}
}

sub cmd2arr { # prevedeme vystup prikazu na pole
	my $cmd = shift;
	my(@arr);
	my $i = 0;
	if(open(POINT,"$cmd |")) {
		while(my $radek = <POINT>) {
			$arr[$i] = $radek;
			$i++;
		}
		close(POINT);
		return @arr;
	} else {
		print STDERR "\nCannot run '$cmd'\n";
		exit;
	}
}

sub ask { # obecna funkce na vstup odpovedi od uzivatele
	my($quest,$def) = @_;
	print "\n$quest [$def] : ";
	my $var = <STDIN>;
	chomp($var);
	if($var eq "") {
		$var = $def;
	}
	return($var);
}

sub sec2hrs {
	my $sec	= shift;
	my $h	= int($sec/3600);
	my $m	= int(($sec-$h*3600)/60);
	my $s	= int($sec-$h*3600-$m*60);
	my $t	= sprintf("%02d:%02d:%02d",$h,$m,$s);
	return $t;
}

sub sec2hrsd {
	my $sec	= shift;
	my $h	= int($sec/3600);
	my $m	= int(($sec-$h*3600)/60);
	my $s	= int($sec-$h*3600-$m*60);
	my @d	= split(/\./,$sec); my $d = $d[1];
	my $t	= sprintf("%02d:%02d:%02d.%03d",$h,$m,$s,$d);
	return $t;
}

1;
